@@ -34,7 +34,7 @@
 
 /* RCS version string for later identification using rcs ident {{{*/
 #ifndef lint
-static volatile const char *RCSid = "@(#) $Id: cfg.c,v 1.17 2002/10/15 10:49:28 mbr Exp $";
+static volatile const char *RCSid = "@(#) $Id: cfg.c,v 1.22 2002/11/10 20:17:56 mbr Exp $";
 #endif
 /* }}} */
 
@@ -51,6 +51,7 @@ int parse_cfg_file(char *name) { /* {{{ */
 	int argument_buffer_ptr = 0;
 	enum mode mode = MODE_INDENT;
 	config file_cfg = {
+		.daemon = -1,
 		.powermgt = -1,
 		.nvram = NULL,
 		.polltime = -1,
@@ -68,7 +69,9 @@ int parse_cfg_file(char *name) { /* {{{ */
 		.osdfont = NULL,
 		.osdcolor = NULL,
 		.osdtimeout = -1,
-		.osdoffset = -1,
+		.osdshadow = -1,
+		.osdvertical = -1,
+		.osdhorizontal = -1,
 		.osdpos = -1,
 		.osdalign = -1
 #endif /* HAVE_LIBXOSD */
@@ -203,7 +206,9 @@ void init_cfg(void) { /* {{{ */
 		_exit(1);
 	}
 	cfg.osdtimeout = DEFAULT_OSDTIMEOUT;
-	cfg.osdoffset = DEFAULT_OSDOFFSET;
+	cfg.osdshadow = DEFAULT_OSDSHADOW;
+	cfg.osdvertical = DEFAULT_OSDVERTICAL;
+	cfg.osdhorizontal = DEFAULT_OSDHORIZONTAL;
 	cfg.osdpos = DEFAULT_OSDPOS;
 	cfg.osdalign = DEFAULT_OSDALIGN;
 #endif /* HAVE_LIBXOSD */
@@ -213,6 +218,10 @@ void init_cfg(void) { /* {{{ */
 
 void override_cfg(config *master) { /* {{{ */
 	/* override the defaults if the are set in master struct */
+	if(master->daemon != -1) {
+		cfg.daemon = master->daemon;
+	}
+
 	if(master->powermgt != -1) {
 		cfg.powermgt = master->powermgt;
 	}
@@ -305,8 +314,16 @@ void override_cfg(config *master) { /* {{{ */
 		cfg.osdtimeout = master->osdtimeout;
 	}
 
-	if(master->osdoffset != -1) {
-		cfg.osdoffset = master->osdoffset;
+	if(master->osdshadow != -1) {
+		cfg.osdshadow = master->osdshadow;
+	}
+
+	if(master->osdvertical != -1) {
+		cfg.osdvertical = master->osdvertical;
+	}
+
+	if(master->osdhorizontal != -1) {
+		cfg.osdhorizontal = master->osdhorizontal;
 	}
 
 	if(master->osdpos != -1) {
@@ -485,13 +502,41 @@ void set_value(char *key, char *arg, config *cfg) { /* {{{ */
 	} /* }}} */
 
 	else if (strcmp(CFG_OSDOFFSET, key) == 0) { /* {{{ */
-		cfg->osdoffset = strtol(arg, &endptr, 10);
+		cfg->osdvertical = strtol(arg, &endptr, 10);
 		if(strlen(endptr) > 0) {
 			fprintf(stderr, "Illegal xosd offset \"%s\"\n", arg);
 			_exit(1);
 		}
 	} /* }}} */
 
+	else if (strcmp(CFG_OSDSHADOW, key) == 0) { /* {{{ */
+		cfg->osdshadow = strtol(arg, &endptr, 10);
+		if(strlen(endptr) > 0) {
+			fprintf(stderr, "Illegal xosd shadow offset \"%s\"\n", arg);
+			_exit(1);
+		}
+	} /* }}} */
+
+	else if (strcmp(CFG_OSDVERTICAL, key) == 0) { /* {{{ */
+		cfg->osdvertical = strtol(arg, &endptr, 10);
+		if(strlen(endptr) > 0) {
+			fprintf(stderr, "Illegal xosd vertical offset \"%s\"\n", arg);
+			_exit(1);
+		}
+	} /* }}} */
+
+	else if (strcmp(CFG_OSDHORIZONTAL, key) == 0) { /* {{{ */
+#ifdef HAVE_LIBXOSD2
+		cfg->osdhorizontal = strtol(arg, &endptr, 10);
+		if(strlen(endptr) > 0) {
+			fprintf(stderr, "Illegal xosd horizontal offset \"%s\"\n", arg);
+			_exit(1);
+		}
+#else
+		fprintf(stderr, "Sorry, OSDHORIZONTAL is only supported by xosd 2.0.0 and above.\n");
+#endif
+	} /* }}} */
+
 	else if (strcmp(CFG_OSDPOS, key) == 0) { /* {{{ */
 		int i = 0;
 		while(arg[i]) {
@@ -502,12 +547,21 @@ void set_value(char *key, char *arg, config *cfg) { /* {{{ */
 			cfg->osdpos = XOSD_top;
 		}
 		else {
-			if(strcmp(CFG_OSDPOS_BOTTOM, arg) == 0) {
-				cfg->osdpos = XOSD_bottom;
+			if(strcmp(CFG_OSDPOS_MIDDLE, arg) == 0) {
+#ifdef HAVE_LIBXOSD2
+				cfg->osdpos = XOSD_middle;
+#else
+				fprintf(stderr, "Sorry, OSDPOS MIDDLE is only supported by xosd 2.0.0 and above.\n");
+#endif
 			}
 			else {
-				fprintf(stderr, "Illegal xosd position %s\n", arg);
-				_exit(1);
+				if(strcmp(CFG_OSDPOS_BOTTOM, arg) == 0) {
+					cfg->osdpos = XOSD_bottom;
+				}
+				else {
+					fprintf(stderr, "Illegal xosd position %s\n", arg);
+					_exit(1);
+				}
 			}
 		}
 	} /* }}} */
@@ -563,6 +617,7 @@ void set_value(char *key, char *arg, config *cfg) { /* {{{ */
 
 void parse_option(int argc, char **argv) { /* {{{ */
 	config cmd_cfg = {
+		.daemon = -1,
 		.powermgt = -1,
 		.nvram = NULL,
 		.polltime = -1,
@@ -580,7 +635,7 @@ void parse_option(int argc, char **argv) { /* {{{ */
 		.osdfont = NULL,
 		.osdcolor = NULL,
 		.osdtimeout = -1,
-		.osdoffset = -1,
+		.osdvertical = -1,
 		.osdpos = -1,
 		.osdalign = -1
 #endif /* HAVE_LIBXOSD */
@@ -591,6 +646,7 @@ void parse_option(int argc, char **argv) { /* {{{ */
 		int next_option;
 		static struct option long_options[] = {
 			{"help",     0, NULL, 'h'},
+			{"daemon",   0, NULL, 'd'},
 			{"powermgt", 1, NULL, 'P'},
 			{"config",   1, NULL, 'c'},
 			{"callback", 1, NULL, 'C'},
@@ -608,9 +664,9 @@ void parse_option(int argc, char **argv) { /* {{{ */
 		};
 
 #ifdef HAVE_LIBXOSD
-		next_option = getopt_long (argc, argv, "hP:c:C:m:o:p:t:H:S:M:v", long_options, &option_index);
+		next_option = getopt_long (argc, argv, "hdP:c:C:m:o:p:t:H:S:M:v", long_options, &option_index);
 #else
-		next_option = getopt_long (argc, argv, "hP:c:C:m:p:t:H:S:M:v", long_options, &option_index);
+		next_option = getopt_long (argc, argv, "hdP:c:C:m:p:t:H:S:M:v", long_options, &option_index);
 #endif /* HAVE_LIBXOSD */
 
 		if (next_option == -1) {
@@ -623,6 +679,10 @@ void parse_option(int argc, char **argv) { /* {{{ */
 				_exit(1);
 				break;
 
+			case 'd':
+				cmd_cfg.daemon = STATE_ON;
+				break;
+
 			case 'P':
 				set_value(CFG_POWERMGT, optarg, &cmd_cfg);
 				break;
@@ -686,10 +746,11 @@ void parse_option(int argc, char **argv) { /* {{{ */
 
 void print_usage(const char *prog) /* {{{ */
 {
-	printf("%s 0.1 (C) 2002 Markus Braun \n\n", prog);
+	printf("%s " VERSION " (C) 2002 Markus Braun \n\n", prog);
 	printf("Usage: %s options\n\n", prog);
 	printf("options:\n");
 	printf("   -h, --help           display this help\n");
+	printf("   -d, --daemon         start in daemon mode\n");
 	printf("   -c, --config=FILE    read FILE as additional configuration file\n");
 	printf("   -m, --mixer=STATE    use OSS software mixer [off]\n");
 #ifdef HAVE_LIBXOSD
