@@ -41,7 +41,7 @@
 
 /* RCS version string for later identification using rcs ident {{{*/
 #ifndef lint
-static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.28 2002/11/04 11:53:59 mbr Exp $";
+static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.33 2002/11/18 14:43:41 mbr Exp $";
 #endif
 /* }}} */
 
@@ -51,6 +51,7 @@ extern config cfg;
 
 /* function prototypes {{{ */
 int get_state(t_nvram_state *nvram_state);
+int get_apm_state(t_nvram_state *nvram_state);
 int fork_app(char * cmd);
 void set_volume_level(void);
 int change_volume(int change);
@@ -60,11 +61,12 @@ int apmiser_running(void);
 int main(int argc, char **argv)/* {{{ */
 {
 	t_nvram_state nvram_state, last_nvram_state;
+	int filedes;
 	int vol = 0;
 	int display_state = 1;
 	char *home;
 	char *cfg_file;
-	char *msg;
+	char *msg = "";
 	char callbackcmd[CALLBACKCMDLENGTH];
 	struct sigaction signal_action;
 #ifdef HAVE_LIBXOSD
@@ -100,19 +102,132 @@ int main(int argc, char **argv)/* {{{ */
 	/* parse options */
 	parse_option(argc, argv);
 
+	/* become a daemon if requested {{{ */
+	if(cfg.daemon == STATE_ON) {
+		/* code inspired by comp.unix.programmer FAQ */
+		switch (fork())
+		{
+			case 0:
+				break;
+			case -1:
+				fprintf(stderr, "Unable to fork daemon");
+				perror(NULL);
+				return -1;
+			default:
+				_exit(0);          /* exit the original process */
+		}
+
+		/* create a new session */
+		if (setsid() < 0) {
+			return -1;
+		}
+
+		switch (fork())
+		{
+			case 0:
+				break;
+			case -1:
+				fprintf(stderr, "Unable to fork daemon");
+				perror(NULL);
+				return -1;
+			default:
+				_exit(0);
+		}
+
+		if(cfg.verbose == STATE_ON) {
+			puts("Daemon started.");
+		}
+
+
+		/* change to root directory */
+		chdir("/");
+
+		/* close all filedescriptors */
+		for(filedes = 0 ; filedes < sysconf(_SC_OPEN_MAX) ; filedes++) {
+			close(filedes);
+		}
+		
+		/* open /dev/null as file descriptor 0 */
+		open("/dev/null",O_RDWR);
+
+		/* duplicate file descriptor 0 to file descriptor 1 (next free) */
+		dup(0);
+
+		/* duplicate file descriptor 0 to file descriptor 2 (next free) */
+		dup(0);
+	} /* }}} */
+
 	/* initialize osd {{{ */
 #ifdef HAVE_LIBXOSD
 	if(cfg.osd == STATE_ON) {
 #ifdef HAVE_LIBXOSD0
-		osd_ptr = xosd_init(cfg.osdfont, cfg.osdcolor, cfg.osdtimeout, cfg.osdpos, cfg.osdoffset, 0);
+		osd_ptr = xosd_init(DEFAULT_OSDFONT, DEFAULT_OSDCOLOR, DEFAULT_OSDTIMEOUT,
+												DEFAULT_OSDPOS, DEFAULT_OSDVERTICAL, DEFAULT_OSDSHADOW);
 #elif HAVE_LIBXOSD1
-		osd_ptr = xosd_init(cfg.osdfont, cfg.osdcolor, cfg.osdtimeout, cfg.osdpos, cfg.osdoffset, 0, 2);
+		osd_ptr = xosd_init(DEFAULT_OSDFONT, DEFAULT_OSDCOLOR, DEFAULT_OSDTIMEOUT,
+												DEFAULT_OSDPOS, DEFAULT_OSDVERTICAL, DEFAULT_OSDSHADOW, DEFAULT_OSDLINES);
+#elif HAVE_LIBXOSD2
+		osd_ptr = xosd_create(DEFAULT_OSDLINES);
 #endif
 		if(osd_ptr == NULL) {
-			fprintf(stderr, "Unable to initialize xosd. Running without onsceen display\n");
+			fprintf(stderr, "Unable to initialize xosd. Running without onsceen display.\n");
 		}
 		else {
-			xosd_set_align(osd_ptr, cfg.osdalign);
+			if(xosd_set_font(osd_ptr, cfg.osdfont)) {
+				fprintf(stderr, "Invalid font \"%s\" for xosd. Running with the default \"%s\".\n",
+								cfg.osdfont, DEFAULT_OSDFONT);
+				xosd_set_font(osd_ptr, DEFAULT_OSDFONT);
+			}
+			
+			if(xosd_set_colour(osd_ptr, cfg.osdcolor)) {
+				fprintf(stderr, "Invalid color \"%s\" for xosd. Running with the default \"%s\".\n",
+								cfg.osdcolor, DEFAULT_OSDCOLOR);
+				xosd_set_colour(osd_ptr, DEFAULT_OSDCOLOR);
+			}
+
+			if(xosd_set_timeout(osd_ptr, cfg.osdtimeout)) {
+				fprintf(stderr, "Invalid timeout \"%d\" for xosd. Running with the default \"%d\".\n",
+								cfg.osdtimeout, DEFAULT_OSDTIMEOUT);
+				xosd_set_timeout(osd_ptr, DEFAULT_OSDTIMEOUT);
+			}
+
+			if(xosd_set_pos(osd_ptr, cfg.osdpos)) {
+				fprintf(stderr, "Invalid position \"%d\" for xosd. Running with the default \"%d\".\n",
+								cfg.osdpos, DEFAULT_OSDPOS);
+				xosd_set_pos(osd_ptr, DEFAULT_OSDPOS);
+			}
+
+#ifdef HAVE_LIBXOSD2
+			if(xosd_set_horizontal_offset(osd_ptr, cfg.osdhorizontal)) {
+				fprintf(stderr, "Invalid horizontal offset \"%d\" for xosd. Running with the default \"%d\".\n",
+								cfg.osdhorizontal, DEFAULT_OSDHORIZONTAL);
+				xosd_set_horizontal_offset(osd_ptr, DEFAULT_OSDHORIZONTAL);
+			}
+
+			if(xosd_set_vertical_offset(osd_ptr, cfg.osdvertical)) {
+				fprintf(stderr, "Invalid vertical offset \"%d\" for xosd. Running with the default \"%d\".\n",
+								cfg.osdvertical, DEFAULT_OSDVERTICAL);
+				xosd_set_vertical_offset(osd_ptr, DEFAULT_OSDVERTICAL);
+			}
+#else
+			if(xosd_set_offset(osd_ptr, cfg.osdvertical)) {
+				fprintf(stderr, "Invalid vertical offset \"%d\" for xosd. Running with the default \"%d\".\n",
+								cfg.osdvertical, DEFAULT_OSDVERTICAL);
+				xosd_set_offset(osd_ptr, DEFAULT_OSDVERTICAL);
+			}
+#endif
+
+			if(xosd_set_shadow_offset(osd_ptr, cfg.osdshadow)) {
+				fprintf(stderr, "Invalid shadow offset \"%d\" for xosd. Running with the default \"%d\".\n",
+								cfg.osdshadow, DEFAULT_OSDSHADOW);
+				xosd_set_shadow_offset(osd_ptr, DEFAULT_OSDSHADOW);
+			}
+
+			if(xosd_set_align(osd_ptr, cfg.osdalign)) {
+				fprintf(stderr, "Invalid alignment \"%d\" for xosd. Running with default \"%d\".\n",
+								cfg.osdalign, DEFAULT_OSDALIGN);
+				xosd_set_align(osd_ptr, DEFAULT_OSDALIGN);
+			}
 		}
 	}
 #endif /* HAVE_LIBXOSD */ /* }}} */
@@ -556,13 +671,92 @@ int get_state(t_nvram_state *nvram_state)/*{{{*/
 	nvram_state->volume_level      = ( buffer[0x60] & 0x0f);
 	nvram_state->volume_toggle     = ( buffer[0x60] & 0x80) >> 7;
 	nvram_state->mute_toggle       = ( buffer[0x60] & 0x40) >> 6;
-	nvram_state->ac_state          = ( buffer[0x57] & 0x10) >> 4;
 	nvram_state->powermgt_ac       = ( buffer[0x39] & 0x07);
 	nvram_state->powermgt_battery  = ( buffer[0x39] & 0x38) >> 3;
 
+	get_apm_state(nvram_state);
+
 	return 0;
 }/*}}}*/
 
+int get_apm_state(t_nvram_state *nvram_state) {  /* {{{ */
+	int i;
+	int proc_apm;
+	char buffer[38];
+	char *tokens[9];
+
+	/* Read the state of the ac line from proc filesystem.
+	 * Documentation of /proc/apm from linux kernel (/usr/src/linux/arch/i386/kernel/apm.c)
+	 * 
+	 * 0) Linux driver version (this will change if format changes)
+	 * 1) APM BIOS Version.  Usually 1.0, 1.1 or 1.2.              
+	 * 2) APM flags from APM Installation Check (0x00):            
+	 *    bit 0: APM_16_BIT_SUPPORT                                
+	 *    bit 1: APM_32_BIT_SUPPORT                                
+	 *    bit 2: APM_IDLE_SLOWS_CLOCK                              
+	 *    bit 3: APM_BIOS_DISABLED                                 
+	 *    bit 4: APM_BIOS_DISENGAGED                               
+	 * 3) AC line status                                           
+	 *    0x00: Off-line                                           
+	 *    0x01: On-line                                            
+	 *    0x02: On backup power (BIOS >= 1.1 only)                 
+	 *    0xff: Unknown                                            
+	 * 4) Battery status                                           
+	 *    0x00: High                                               
+	 *    0x01: Low                                                
+	 *    0x02: Critical                                           
+	 *    0x03: Charging                                           
+	 *    0x04: Selected battery not present (BIOS >= 1.2 only)    
+	 *    0xff: Unknown                                            
+	 * 5) Battery flag                                             
+	 *    bit 0: High                                              
+	 *    bit 1: Low                                               
+	 *    bit 2: Critical                                          
+	 *    bit 3: Charging                                          
+	 *    bit 7: No system battery                                 
+	 *    0xff: Unknown                                            
+	 * 6) Remaining battery life (percentage of charge):           
+	 *    0-100: valid                                             
+	 *    -1: Unknown                                              
+	 * 7) Remaining battery life (time units):                     
+	 *    Number of remaining minutes or seconds                   
+	 *    -1: Unknown                                              
+	 * 8) min = minutes; sec = seconds */                          
+
+	 /* open /proc/apm */
+	 if((proc_apm = open("/proc/apm", O_RDONLY)) == -1) {
+		 return -1;
+	 }
+
+	 /* read apm state */
+	 if(read(proc_apm, buffer, sizeof(buffer)) != sizeof(buffer)) {
+		 close(proc_apm);
+		 return -1;
+	 }
+
+	 /* no need to keep file open */
+	 close(proc_apm);
+
+	 /* tokenize the apm string */
+	 tokens[0] = strtok(buffer, " ");
+	 for(i = 1 ; i < sizeof(tokens)/sizeof(char*) ; i++) {
+		 tokens[i] = strtok(NULL, " ");
+	 }
+
+	 /* determine the AC line status */
+	 switch(strtol(tokens[3], NULL, 16)) {
+		 case 0x00:
+			 nvram_state->ac_state = STATE_OFF;
+			 break;
+
+		 case 0x01:
+			 nvram_state->ac_state = STATE_ON;
+			 break;
+	 }
+
+	 return 0;
+} /* }}} */
+	
 int fork_app(char * cmd) { /* {{{ */
 	//char *args[4];
 	//args[0] = "sh";
